package KDDCup05.termstools;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Scanner;

import KDDCup05.termstools.stem.KrovetzStemmer;
import KDDCup05.termstools.stem.Stemmer;
import KDDCup05.termstools.stem.res.ResourceProducer;

import lemurproject.indri.DocumentVector;
import lemurproject.indri.QueryEnvironment;
import lemurproject.indri.ScoredExtentResult;

public class ResultOfAParticularTermNew {

	public static enum RetrievalCutOff{
		THRESHOLD,
		ANDDF,
		ANDDFSEED,
		MINDF,
		MINDFSEED,
		GDF // Den xrhsimopoieitai!
	}

	public TermInfoNew Term;
	//private KTermPhrase stemTerm;
	public ArrayList<TermInfoNew> RelevantTerms;
	private int M;

	public ResultOfAParticularTermNew(String term) {
		this.Term = new TermInfoNew();
		this.Term.term = term;
		//stemTerm = new KTermPhrase();
		this.RelevantTerms = new ArrayList<TermInfoNew>();
		this.M = 0;
	}

	public ResultOfAParticularTermNew() {
		this.Term = new TermInfoNew();
		//stemTerm = new KTermPhrase();
		this.RelevantTerms = new ArrayList<TermInfoNew>();
		this.M = 0;
	}

	public boolean readTermInfoFromFile(int ThresholdOfTerms, String bagWithScrambleTerms){
		try {

			BufferedReader in = new BufferedReader(new InputStreamReader(new FileInputStream(bagWithScrambleTerms), "UTF-8"));

			String inputLine;
			int count =0;
			while ((inputLine = in.readLine()) != null) {
				TermInfoNew curr = new TermInfoNew();
				if(curr.parseTermInfo(inputLine)){
					if(count == 0){
						this.Term = curr;
						count++;
					}else{
						this.RelevantTerms.add(curr);
						count++;
					}
					if(ThresholdOfTerms > 0 && count-1 >= ThresholdOfTerms) break;
				}
			}
		}catch(Exception e){
			System.err.println("ERROR when reading bag with Scramble terms and DocIDs:" + e.toString());
			return false;
		}	
		return true;
	}

	public boolean readTermInfoFromFileWithPrivacyFilters(int ThresholdOfTerms, String bagWithScrambleTerms, double k, double level, int num_of_docs_in_index){
		try {
			this.M = num_of_docs_in_index;
			BufferedReader in = new BufferedReader(new InputStreamReader(new FileInputStream(bagWithScrambleTerms), "UTF-8"));

			String inputLine;
			int count =0;
			while ((inputLine = in.readLine()) != null) {
				TermInfoNew curr = new TermInfoNew();
				if(curr.parseTermInfo(inputLine)){
					if(count == 0){
						this.Term = curr;
						count++;
					}else{
						if(curr.PrivacyKFilter(k) && curr.PrivacyLevelFilter(level, this.Term.DF, this.M)){
							this.RelevantTerms.add(curr);
							count++;
						}
					}
					if(ThresholdOfTerms > 0 && count-1 >= ThresholdOfTerms) break;
				}
			}
		}catch(Exception e){
			System.err.println("ERROR when reading bag with Scramble terms and DocIDs:" + e.toString());
			return false;
		}	
		return true;
	}
	
	public boolean readTermInfoFromFileWithPrivacyFiltersAndUpdateDocIDs(int ThresholdOfTerms, String index, String bagWithScrambleTerms, double k, double level, int num_of_docs_in_index){
		try {
			QueryEnvironment qenv = new QueryEnvironment();
			qenv.addIndex(index);

			//this.M = (int)qenv.documentCount();
			this.M = num_of_docs_in_index;
			
			BufferedReader in = new BufferedReader(new InputStreamReader(new FileInputStream(bagWithScrambleTerms), "UTF-8"));

			String inputLine;
			int count =0;
			while ((inputLine = in.readLine()) != null) {
				TermInfoNew curr = new TermInfoNew();
				if(curr.parseTermInfo(inputLine)){
					if(count == 0){
						this.Term = curr;
						this.Term.DocIDs = getDocuments(qenv, this.Term.term.replace("_", " "), RetrievalCutOff.THRESHOLD, this.Term.DF);
						if(this.Term.DocIDs.size() != this.Term.DF) System.err.println("Error in Seed! Don't match the numbers!");
						count++;
					}else{
						if(curr.PrivacyKFilter(k) && curr.PrivacyLevelFilter(level, this.Term.DF, this.M)){
							count++;
						}
						curr.DocIDs = getDocuments(qenv, curr.term.replace("_", " "), RetrievalCutOff.THRESHOLD, curr.DF);
						if(!curr.updateMatchesWithSeedQueryInDF(this.Term.DocIDs)) System.err.println("Error in scramble query! Don't match the numbers in '"+curr.term+"'!");
						this.RelevantTerms.add(curr);
					}
					if(ThresholdOfTerms > 0 && count-1 >= ThresholdOfTerms) break;
				}
			}
		}catch(Exception e){
			System.err.println("ERROR when reading bag with Scramble terms and DocIDs:" + e.toString());
			return false;
		}	
		return true;
	}

	public boolean readTermInfoFromFileWithPrivacyFiltersAndMIFilter(int ThresholdOfTerms, String bagWithScrambleTerms, double k, double level, double mi, int num_of_docs_in_index){
		try {
			this.M = num_of_docs_in_index;
			BufferedReader in = new BufferedReader(new InputStreamReader(new FileInputStream(bagWithScrambleTerms), "UTF-8"));

			String inputLine;
			int count =0;
			while ((inputLine = in.readLine()) != null) {
				TermInfoNew curr = new TermInfoNew();
				if(curr.parseTermInfo(inputLine)){
					if(curr.score < mi && count > ThresholdOfTerms) break;
					if(count == 0){
						this.Term = curr;
						count++;
					}else{
						if(curr.PrivacyKFilter(k) && curr.PrivacyLevelFilter(level, this.Term.DF, this.M)){
							this.RelevantTerms.add(curr);
							count++;
						}
					}
					//if(ThresholdOfTerms > 0 && count-1 >= ThresholdOfTerms) break;
				}
			}
		}catch(Exception e){
			System.err.println("ERROR when reading bag with Scramble terms and DocIDs:" + e.toString());
			return false;
		}	
		return true;
	}

	public boolean retrievEverythingFromIndexSingleAndKWordsPerDocTerms(String index, String squery, int num_of_terms, int threshold, int capacity, RetrievalCutOff rco){
		try {
			QueryEnvironment qenv = new QueryEnvironment();
			qenv.addIndex(index);
			//qenv.setScoringRules(new String[]{"-baseline=tfidf"}); // Blakies den douleuei!!!

			this.M = (int)qenv.documentCount();

			HashSet<String> stops = getStopWords();

			// Cleaning
			squery = squery.trim();
			while (squery.contains("  ")){
				squery = squery.replace("  ", " ");
			}

			///////////////////////////////////////////////////////////////////////

			this.Term.DocIDsMinDF = getDocuments(qenv, squery, rco, -1);
			if(this.Term.DocIDsMinDF.size() < threshold){
				this.Term.DocIDs = getDocuments(qenv, squery, RetrievalCutOff.THRESHOLD, this.Term.DocIDsMinDF.size());
			}else{
				this.Term.DocIDs = getDocuments(qenv, squery, RetrievalCutOff.THRESHOLD, threshold);
			}
			this.Term.score = 1.0000;
			this.Term.term = squery.replace(" ", "_");
			this.Term.hints = this.Term.DocIDs.size();
			this.Term.hintsMinDF = this.Term.DocIDsMinDF.size();
			this.Term.DF = this.Term.DocIDsMinDF.size(); //Einai sigouro? O Avi ti leei? Apanthsh: Einai ok!!!

			//////////////////////////////////////////////////////////////////////

			// Allow the seed terms
			stops.removeAll(getStemmedKrovetzTermsOfQuery(new String(squery)));
			//this.stemTerm.kterms = getStemmedKrovetzTermsOfQuery(squery);

			//////////////////////////////////////////////////////////////////////

			int[] ids = new int[this.Term.hints];
			for(int i = 0; i< this.Term.hints; i++){
				ids[i] = this.Term.DocIDs.get(i);
			}
			DocumentVector[] dv = qenv.documentVectors(ids);

			MySortedHashMap h = new MySortedHashMap(capacity);

			for (int i = 0; i < dv.length; i++) {

				String[] t = dv[i].stems;

				findScrambleMultiTermsPerDoc(qenv, stops, num_of_terms, t, h);

				for(int jj=1; jj< t.length;jj++){ //Prosoxh to jj einai 1 giati sto 0 einai panta to [OOV]
					KTermPhrase kt = new KTermPhrase();
					kt.add(t[jj]);
					if(!stops.contains(t[jj]) && !h.containsKey(kt) /*&& !kt.contains(this.stemTerm)*/){
						TermInfoNew ti = new TermInfoNew(t[jj]);		
						ti.DocIDs = getDocuments(qenv, t[jj], null,-1); //getDocuments(qenv, t[jj], RetrievalCutOff.ANDDF,-1);
						ti.DF = ti.DocIDs.size();
						ti.keepOnlyMatchesWithSeedQuery(this.Term.DocIDs, this.Term.DocIDsMinDF);
						if(ti.DF > 0 && ti.hints > 0){
							ti.score = ti.calcMI(this.Term.hints, this.M);
							if( ti.score >= 0.9999){
								ti.score = 0.9999;
							}
							h.put(kt, ti);
						}
					}
				}
			}

			//////////////////////////////////////////////////////////////////////

			this.RelevantTerms = h.getArrayListWithScrambleQueries();

			qenv.close();
		}catch(Exception e){
			System.err.println("ERROR With getting Scramble Terms from index: " + e.toString());
			e.printStackTrace();
			return false;
		}	
		return true;
	}

	public void findScrambleMultiTermsPerDoc(QueryEnvironment qenv, HashSet<String> stops, int num_of_terms, String[] t, MySortedHashMap h){
		if(num_of_terms > 1){
			ArrayList<String> win = new ArrayList<String>();
			ArrayList<KTermPhrase> kte = new ArrayList<KTermPhrase>();
			for(int j=1; j < t.length; j++){ //Prosoxh to j einai 1 giati sto 0 einai panta to [OOV]
				if(!stops.contains(t[j])){
					win.add(t[j]);
					kte.add(new KTermPhrase(t[j]));
				}
			}

			for(int n = 1; n < num_of_terms;n++){
				kte = loopPerDoc(kte,win,n, qenv, h);
			}
		}
	}

	public ArrayList<KTermPhrase> loopPerDoc(ArrayList<KTermPhrase> kterms, ArrayList<String> win, int pos, QueryEnvironment qenv, MySortedHashMap h){
		ArrayList<KTermPhrase> kw = new ArrayList<KTermPhrase>();
		for(int k=0;k<kterms.size();k++){
			for(int m=k+pos;m<win.size();m++){
				KTermPhrase kt = (KTermPhrase)kterms.get(k).clone();
				kt.add(win.get(m));
				//if(!kt.contains(this.stemTerm)){
				if(!h.containsKey(kt)){
					TermInfoNew ti = new TermInfoNew(kt.toString());
					//System.out.println(ti.term);
					ti.DocIDs = getDocuments(qenv, kt.toString().replace("_", " "), RetrievalCutOff.ANDDF,-1);
					ti.DF = ti.DocIDs.size();
					ti.keepOnlyMatchesWithSeedQuery(this.Term.DocIDs, this.Term.DocIDsMinDF);
					if(ti.DF > 0){
						if(ti.hints > 0){
							ti.score = ti.calcMI(this.Term.hints, this.M);
							if( ti.score >= 0.9999){
								ti.score = 0.9999;
							}
							h.put(kt, ti);
						}
						kw.add(kt); // Logo andDF
					}
				}else{
					kw.add(kt);
				}			
				//}
			}
		}
		return kw;
	}

	public boolean retrievEverythingFromIndexSingleAndKWordsWithWindow(String index, String squery, int window, int num_of_terms, int threshold, RetrievalCutOff rco){
		try {
			QueryEnvironment qenv = new QueryEnvironment();
			qenv.addIndex(index);
			//qenv.setScoringRules(new String[]{"-baseline=tfidf"}); // Blakies den douleuei!!!

			this.M = (int)qenv.documentCount();

			HashSet<String> stops = getStopWords();

			// Cleaning
			squery = squery.trim();
			while (squery.contains("  ")){
				squery = squery.replace("  ", " ");
			}

			///////////////////////////////////////////////////////////////////////

			this.Term.DocIDsMinDF = getDocuments(qenv, squery, rco, -1);
			if(this.Term.DocIDsMinDF.size() < threshold){
				this.Term.DocIDs = getDocuments(qenv, squery, RetrievalCutOff.THRESHOLD, this.Term.DocIDsMinDF.size());
			}else{
				this.Term.DocIDs = getDocuments(qenv, squery, RetrievalCutOff.THRESHOLD, threshold);
			}
			this.Term.score = 1.0000;
			this.Term.term = squery.replace(" ", "_");
			this.Term.hints = this.Term.DocIDs.size();
			this.Term.hintsMinDF = this.Term.DocIDsMinDF.size();
			this.Term.DF = this.Term.DocIDsMinDF.size(); //Einai sigouro? O Avi ti leei? Apanthsh: Einai ok!!!

			//////////////////////////////////////////////////////////////////////

			// Allow the seed terms
			stops.removeAll(getStemmedKrovetzTermsOfQuery(new String(squery)));
			//this.stemTerm.kterms = getStemmedKrovetzTermsOfQuery(squery);

			//////////////////////////////////////////////////////////////////////

			int[] ids = new int[this.Term.hints];
			for(int i = 0; i< this.Term.hints; i++){
				ids[i] = this.Term.DocIDs.get(i);
			}
			DocumentVector[] dv = qenv.documentVectors(ids);

			HashMap<KTermPhrase,TermInfoNew> h = new HashMap<KTermPhrase,TermInfoNew>();	
			//MySortedHashMap<KTermPhrase,TermInfoNew> h = new MySortedHashMap<KTermPhrase,TermInfoNew>();

			for (int i = 0; i < dv.length; i++) {

				String[] t = dv[i].stems;
				int[] po = dv[i].positions;
				findScrambleMultiTermsInWindow(qenv, stops, window, num_of_terms,  po, t, h);

				for(int jj=1; jj< t.length;jj++){ //Prosoxh to jj einai 1 giati sto 0 einai panta to [OOV]
					KTermPhrase kt = new KTermPhrase();
					kt.add(t[jj]);
					if(!stops.contains(t[jj]) && !h.containsKey(kt) /*&& !kt.contains(this.stemTerm)*/){
						TermInfoNew ti = new TermInfoNew(t[jj]);		
						ti.DocIDs = getDocuments(qenv, t[jj], null,-1); //getDocuments(qenv, t[jj], RetrievalCutOff.ANDDF,-1);
						ti.DF = ti.DocIDs.size();
						ti.keepOnlyMatchesWithSeedQuery(this.Term.DocIDs, this.Term.DocIDsMinDF);
						if(ti.DF > 0 && ti.hints > 0){
							ti.score = ti.calcMI(this.Term.hints, this.M);
							if( ti.score >= 0.9999){
								ti.score = 0.9999;
							}
							h.put(kt, ti);
						}
					}
				}
			}

			//////////////////////////////////////////////////////////////////////


			/* Convert HashMap to ArrayList and sort by score*/
			Iterator<TermInfoNew> it = h.values().iterator();
			while(it.hasNext()){
				this.RelevantTerms.add(it.next());
				//TermInfoNew ti = it.next();
				//if(ti.DF > 0 && ti.hints > 0){ //// Einai kolpo gia na mhn emfanizei kati koula opws [OOV]
				//ti.score = ti.calcMI(this.Term.hints, M);
				//this.RelevantTerms.add(ti);
				//}
			}
			//sortRelevantTermsBy(TermInfoNew.ComparingBy.SCORE, false);
			Collections.sort(this.RelevantTerms);

			qenv.close();
		}catch(Exception e){
			System.err.println("ERROR With getting Scramble Terms from index: " + e.toString());
			e.printStackTrace();
			return false;
		}	
		return true;
	}

	public void findScrambleMultiTermsInWindow(QueryEnvironment qenv, HashSet<String> stops, int window, int num_of_terms, int[] po, String[] t, HashMap<KTermPhrase,TermInfoNew> h){
		if(num_of_terms > 1){
			for(int i=0; i< po.length-window; i++){
				ArrayList<String> win = new ArrayList<String>();
				ArrayList<KTermPhrase> kte = new ArrayList<KTermPhrase>();
				for(int j=i; j< i+window; j++){
					if(!stops.contains(t[po[j]]) && !win.contains(t[po[j]])){
						win.add(t[po[j]]);
						kte.add(new KTermPhrase(t[po[j]]));
					}
				}

				for(int n = 1; n < num_of_terms;n++){
					kte = loop(kte,win,n, qenv, h);
				}

				//			for(int k=0;k<win.size();k++){
				//				for(int m=k+1;m<win.size();m++){
				//					//String str1 = win.get(k)+"_"+win.get(m);
				//					//String str2 = win.get(m)+"_"+win.get(k);
				//					if(!h.containsKey(new KTermPhrase(win.get(k)+"_"+win.get(m))) /*&& !h.containsKey(str2)*/){
				//						TermInfoNew ti = new TermInfoNew(win.get(k)+"_"+win.get(m));
				//						//System.out.println(ti.term);
				//						ti.DocIDs = getDocuments(qenv, win.get(k)+" "+win.get(m), RetrievalCutOff.ANDDF,-1);
				//						ti.DF = ti.DocIDs.size();
				//						ti.keepOnlyMatchesWithSeedQuery(this.Term.DocIDs, this.Term.DocIDsMinDF);
				//						h.put(new KTermPhrase(win.get(k)+"_"+win.get(m)), ti);
				//					}
				//				}
				//			}
			}
		}
	}

	public ArrayList<KTermPhrase> loop(ArrayList<KTermPhrase> kterms, ArrayList<String> win, int pos, QueryEnvironment qenv, HashMap<KTermPhrase,TermInfoNew> h){
		ArrayList<KTermPhrase> kw = new ArrayList<KTermPhrase>();
		for(int k=0;k<kterms.size();k++){
			for(int m=k+pos;m<win.size();m++){
				KTermPhrase kt = (KTermPhrase)kterms.get(k).clone();
				kt.add(win.get(m));
				//if(!kt.contains(this.stemTerm)){
				if(!h.containsKey(kt)){
					TermInfoNew ti = new TermInfoNew(kt.toString());
					//System.out.println(ti.term);
					ti.DocIDs = getDocuments(qenv, kt.toString().replace("_", " "), RetrievalCutOff.ANDDF,-1);
					ti.DF = ti.DocIDs.size();
					ti.keepOnlyMatchesWithSeedQuery(this.Term.DocIDs, this.Term.DocIDsMinDF);
					if(ti.DF > 0){
						if(ti.hints > 0){
							ti.score = ti.calcMI(this.Term.hints, this.M);
							if( ti.score >= 0.9999){
								ti.score = 0.9999;
							}
							h.put(kt, ti);
						}
						kw.add(kt);  // Logo andDF to bazw mesa sto if
					}
				}else{
					kw.add(kt);
				}
				//}
			}
		}
		return kw;
	}

	public void sortHashMapByValueAndPrint(HashMap<?, Integer> t){

		//Transfer as List and sort it
		ArrayList<Map.Entry<?, Integer>> l = new ArrayList(t.entrySet());
		Collections.sort(l, new Comparator<Map.Entry<?, Integer>>(){

			public int compare(Map.Entry<?, Integer> o1, Map.Entry<?, Integer> o2) {
				return o1.getValue().compareTo(o2.getValue());
			}});

		System.out.println("Size: "+l.size());
		for(int i=l.size()-1;i>=0;i--){
			System.out.println(l.get(i).getKey()+" "+l.get(i).getValue());
		}  
	}

	public ArrayList<Integer> getDocuments(QueryEnvironment qenv, String squery, RetrievalCutOff rco, int threshold){

		ArrayList<Integer> results = new ArrayList<Integer>();
		try {
			if(rco == RetrievalCutOff.GDF){
				//////////////////////////////////////////////////////////
				// Preprocessing
				String[] strs = squery.split(" ");

				// Get minDF
				long minDF = qenv.documentCount(strs[0]);
				for (int i = 0; i < strs.length; i++){
					long tmp = qenv.documentCount(strs[i]);
					if (tmp < minDF) {
						minDF = tmp;
					}
				}

				// Get andDF
				String andquery = "#uw(" + squery + ")";
				//String andquery = "#band(" + squery + ")";
				long andDF = qenv.runQuery(andquery, (int)qenv.documentCount()).length;
				if (andDF == 0) { 
					andDF = 1; 
				}

				// Get gDF
				double gDF = Math.sqrt((double)andDF*(double)minDF);

				// Run query
				//String orquery = "#or(" + squery + ")";
				String orquery = squery;
				ScoredExtentResult[] or_results = qenv.runQuery(orquery, (int)Math.ceil((double)gDF));

				//////////////////////////////////////////////////////////////

				for (int i = 0; i < or_results.length; i++){
					results.add(or_results[i].document);
				}
			}else if(rco == RetrievalCutOff.MINDF){
				//////////////////////////////////////////////////////////
				// Preprocessing
				String[] strs = squery.split(" ");

				// Get minDF
				long minDF = qenv.documentCount(strs[0]);
				for (int i = 0; i < strs.length; i++){
					long tmp = qenv.documentCount(strs[i]);
					if (tmp < minDF) {
						minDF = tmp;
					}
				}

				if (minDF > 0) { 
					// Run query
					//String orquery = "#or(" + squery + ")";
					String orquery = squery;
					ScoredExtentResult[] or_results = qenv.runQuery(orquery, (int)minDF);

					//////////////////////////////////////////////////////////////

					for (int i = 0; i < or_results.length; i++){
						results.add(or_results[i].document);
					}
				}
			}else if(rco == RetrievalCutOff.MINDFSEED){
				//////////////////////////////////////////////////////////
				// Preprocessing
				String[] strs = squery.split(" ");

				// Get minDF
				long minDF = qenv.documentCount(strs[0]);
				for (int i = 0; i < strs.length; i++){
					long tmp = qenv.documentCount(strs[i]);
					if (tmp < minDF) {
						minDF = tmp;
					}
				}
				if (minDF == 0) { //Mpinia gia na mhn exw problhmata sta k kai level
					minDF = 1; 
				}

				// Run query
				//String orquery = "#or(" + squery + ")";
				String orquery = squery;
				ScoredExtentResult[] or_results = qenv.runQuery(orquery, (int)minDF);

				//////////////////////////////////////////////////////////////

				for (int i = 0; i < or_results.length; i++){
					results.add(or_results[i].document);
				}
			}else if(rco == RetrievalCutOff.ANDDF){

				// Get andDF
				String andquery = "#uw(" + squery + ")";
				//String andquery = "#band(" + squery + ")";
				long andDF = qenv.runQuery(andquery, (int)qenv.documentCount()).length;

				//if (andDF == 0) { //Ti na kanw edw? --> To afhnw 0, giati etsi kai alliws h frash 8a einai gia petama
				//	andDF = 1; 
				//}

				if(andDF > 0){
					// Run query
					//String orquery = "#or(" + squery + ")";
					String orquery = squery;
					ScoredExtentResult[] or_results = qenv.runQuery(orquery, (int)andDF);

					//////////////////////////////////////////////////////////////

					for (int i = 0; i < or_results.length; i++){
						results.add(or_results[i].document);
					}
				}
			}else if(rco == RetrievalCutOff.ANDDFSEED){

				// Get andDF
				String andquery = "#uw(" + squery + ")";
				//String andquery = "#band(" + squery + ")";
				long andDF = qenv.runQuery(andquery, (int)qenv.documentCount()).length;

				if (andDF == 0) { //Ti na kanw edw? --> To afhnw 0, giati etsi kai alliws h frash 8a einai gia petama
					andDF = 1; 
				}

				// Run query
				//String orquery = "#or(" + squery + ")";
				String orquery = squery;
				ScoredExtentResult[] or_results = qenv.runQuery(orquery, (int)andDF);

				//////////////////////////////////////////////////////////////

				for (int i = 0; i < or_results.length; i++){
					results.add(or_results[i].document);
				}
			}else if(rco == RetrievalCutOff.THRESHOLD){
				//////////////////////////////////////////////////////////

				// Run query
				//String orquery = "#or(" + squery + ")";
				String orquery = squery;
				ScoredExtentResult[] or_results = qenv.runQuery(orquery, threshold);

				//////////////////////////////////////////////////////////////

				for (int i = 0; i < or_results.length; i++){
					results.add(or_results[i].document);
				}
			}else{
				ScoredExtentResult[] res = qenv.runQuery(squery, (int)qenv.documentCount());

				//////////////////////////////////////////////////////////////

				for (int i = 0; i < res.length; i++){
					results.add(res[i].document);
				}
			}
		}catch(Exception e){
			System.err.println("ERROR With getting Documents IDs 'getDocuments(...)':" + e.toString());
		}

		return results;
	}

	public void sortRelevantTermsBy(TermInfoNew.ComparingBy cb, boolean Ascending){
		int n = this.RelevantTerms.size();
		for (int pass=1; pass < n; pass++) {  // count how many times
			// This next loop becomes shorter and shorter
			for (int i=0; i < n-pass; i++) {
				if(Ascending){
					if (this.RelevantTerms.get(i).compareTo(this.RelevantTerms.get(i+1), cb) > 0) {
						// exchange elements
						TermInfoNew temp = this.RelevantTerms.get(i);  
						this.RelevantTerms.set(i, this.RelevantTerms.get(i+1));
						this.RelevantTerms.set(i+1,temp);
					}
				}else{
					if (this.RelevantTerms.get(i).compareTo(this.RelevantTerms.get(i+1), cb) < 0) {
						// exchange elements
						TermInfoNew temp = this.RelevantTerms.get(i);  
						this.RelevantTerms.set(i, this.RelevantTerms.get(i+1));
						this.RelevantTerms.set(i+1,temp);
					}
				}
			}
		}	
	}

	private ArrayList<String> getStemmedKrovetzTermsOfQuery(String query){
		ArrayList<String> sterms = new ArrayList<String>();
		try {
			Stemmer st = new KrovetzStemmer();
			String[] stermsa = st.stemTextBlock(query, true).split(" ");
			for(String str: stermsa){
				sterms.add(str);
			}
		} catch (Exception e) {
			System.err.println(e.toString());
		}
		return sterms;
	}

	public HashSet<String> getStopWords(){
		HashSet<String> stops = new HashSet<String>();

		//Source: http://code.google.com/p/text-categorization/
		try {
			Scanner in = new Scanner(ResourceProducer.getResourceByName("stopwords-lemur"));
			while (in.hasNext()) {
				stops.add(in.next());
			}

			in = new Scanner(ResourceProducer.getResourceByName("stopwords-tcbook"));
			while (in.hasNext()) {
				stops.add(in.next());
			}

			in = new Scanner(ResourceProducer.getResourceByName("stopwords-web"));
			while (in.hasNext()) {
				stops.add(in.next());
			}

			in.close();
		} catch (Exception e) {
			System.err.println(e.toString());
		}

		//return stops.toArray(new String[stops.size()]); //Convert to String[]
		return stops;
	}

	public int getNumberOfRelevantTerms(){
		return this.RelevantTerms.size();
	}

	public void printAll(int threshold){
		this.Term.print();
		for (int i = 0; i < threshold && i < this.RelevantTerms.size(); i++) {
			this.RelevantTerms.get(i).print();
		}
	}
        
        public String getFirstResultsAsString(int threshold){
            String str = "";
		for (int i = 0; i < threshold && i < this.RelevantTerms.size(); i++) {
                    str += " "+this.RelevantTerms.get(i).term;
		}
            return str;    
	}
        
	public void reCalcDistanceMI(double trunc, double M){
		for (int i = 0; i < this.RelevantTerms.size(); i++) {
			this.RelevantTerms.get(i).score = this.RelevantTerms.get(i).calcDistanceMI(trunc, M);
		}
	}

	public void printAllToFile(String filename, int threshold){	        
		BufferedWriter bufferedWriter = null;      
		try {

			//Construct the BufferedWriter object
			bufferedWriter = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(filename), "UTF-8"));

			bufferedWriter.write(this.Term.toString());
			for (int i = 0; i < threshold && i < this.RelevantTerms.size(); i++) {
				bufferedWriter.newLine();
				bufferedWriter.write(this.RelevantTerms.get(i).toString());
			}      

		} catch (FileNotFoundException ex) {
			ex.printStackTrace();
		} catch (IOException ex) {
			ex.printStackTrace();
		} finally {
			//Close the BufferedWriter
			try {
				if (bufferedWriter != null) {
					bufferedWriter.flush();
					bufferedWriter.close();
				}
			} catch (IOException ex) {
				ex.printStackTrace();
			}
		}
	}
	
	public void printOnlyScramleQueriesToFile(String filename, int threshold){	        
		BufferedWriter bufferedWriter = null;      
		try {

			//Construct the BufferedWriter object
			bufferedWriter = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(filename), "UTF-8"));

			//bufferedWriter.write(this.Term.toString());
			for (int i = 0; i < threshold && i < this.RelevantTerms.size(); i++) {
				bufferedWriter.write(this.RelevantTerms.get(i).term.replace("_", " "));
				bufferedWriter.newLine();
			}      

		} catch (FileNotFoundException ex) {
			ex.printStackTrace();
		} catch (IOException ex) {
			ex.printStackTrace();
		} finally {
			//Close the BufferedWriter
			try {
				if (bufferedWriter != null) {
					bufferedWriter.flush();
					bufferedWriter.close();
				}
			} catch (IOException ex) {
				ex.printStackTrace();
			}
		}
	}

	public Object clone(){
		ResultOfAParticularTermNew ro = new ResultOfAParticularTermNew();
		ro.Term = (TermInfoNew) this.Term.clone();
		ro.RelevantTerms = (ArrayList<TermInfoNew>) this.RelevantTerms.clone();
		ro.M = new Integer(this.M);
		return ro;
	}

}
