package mainPackage;
/*
 * TrayIconCarre.java
 */

import dialogs.AboutJFrame;
import dialogs.GraphJFrame;
import dialogs.SetUserJFrame;
import dialogs.StatisticsJFrame;
import finals.Finals;
import static finals.Finals.checkingPeriodOfRDFSender;
import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.logging.FileHandler;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.*;
import org.tmatesoft.sqljet.browser.DBBrowser;
import scheduler.CategorizerExecutionRunnable;
import scheduler.RDFSenderExecutionRunnable;
import sqlite.MySQLiteHelper;
import tools.Files;
import tools.RefineryUtilities;
import webserver.WServer;

public class TrayIconCarre {

    static MySQLiteHelper db;
    
    static ScheduledExecutorService scheduler = null;

    static WServer wServer;
    
    static DBBrowser br = null;
    static AboutJFrame ab = null;
    public static SetUserJFrame su = null;
    static GraphJFrame gb = null;
    static StatisticsJFrame st = null;
    
    public static void main(String[] args) {
        try {
            Files.createDirectory(Finals.logsFolder);
            Logger.getLogger("").addHandler(new FileHandler(Finals.logs, 1 * 1024 * 1024, 3, true));
        } catch (IOException ex) {
            Logger.getLogger(TrayIconCarre.class.getName()).log(Level.SEVERE, null, ex);
        } catch (SecurityException ex) {
            Logger.getLogger(TrayIconCarre.class.getName()).log(Level.SEVERE, null, ex);
        }
        
        db = MySQLiteHelper.getInstance();
        
        Finals.loadProperties();
        
        //Check to start with windows or not
        if(args.length == 1 && args[0].equals("autostart")){
            if(!Finals.stWithWindows) return;
        }

        wServer = new WServer(db);
        
        scheduler = Executors.newScheduledThreadPool(Finals.threadsOnScheduler);
        scheduler.execute(wServer);
        scheduler.scheduleAtFixedRate(new CategorizerExecutionRunnable(db,Finals.indexForCategorizer), Finals.checkingPeriodOfCategorizer, Finals.checkingPeriodOfCategorizer, TimeUnit.SECONDS);
        scheduler.scheduleAtFixedRate(new RDFSenderExecutionRunnable(db), Finals.checkingPeriodOfCategorizer+(Finals.checkingPeriodOfRDFSender/2), Finals.checkingPeriodOfRDFSender, TimeUnit.SECONDS);
        //scheduler.scheduleAtFixedRate(XXUploader, XXXX, XXXX, TimeUnit.SECONDS); //TODO

        /* Use an appropriate Look and Feel */
        try {
            UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
            //UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
        } catch (UnsupportedLookAndFeelException ex) {
            ex.printStackTrace();
        } catch (IllegalAccessException ex) {
            ex.printStackTrace();
        } catch (InstantiationException ex) {
            ex.printStackTrace();
        } catch (ClassNotFoundException ex) {
            ex.printStackTrace();
        }
        /* Turn off metal's use of bold fonts */
        UIManager.put("swing.boldMetal", Boolean.FALSE);
        //Schedule a job for the event-dispatching thread:
        //adding TrayIcon.
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                createAndShowGUI();
            }
        });
    }

    private static void createAndShowGUI() {
        //Check the SystemTray support
        if (!SystemTray.isSupported()) {
            System.out.println("SystemTray is not supported");
            return;
        }
        final PopupMenu popup = new PopupMenu();
        Finals.trayIcon = new TrayIcon(Finals.putIconImage(), Finals.appName+" ("+Finals.appSubName+") "+Finals.appVersion);
        Finals.trayIcon.setImageAutoSize(true);
        Finals.trayIcon.setToolTip(Finals.appName+"\n("+Finals.appSubName+")");
        final SystemTray tray = SystemTray.getSystemTray();

        // Create a popup menu components
        MenuItem aboutItem = new MenuItem("About");
        aboutItem.setFont(new java.awt.Font("Tahoma", 1, 11));
        MenuItem accountItem = new MenuItem("Set Account");
        MenuItem browseDBItem = new MenuItem("Browse DB");
        MenuItem visualItem = new MenuItem("Visualization of Results");
        MenuItem statsItem = new MenuItem("Statistics");
        CheckboxMenuItem cb1 = new CheckboxMenuItem("Start with Windows");
        if(Finals.stWithWindows){
            cb1.setState(true);
        }else{
            cb1.setState(false);
        }
        //CheckboxMenuItem cb2 = new CheckboxMenuItem("Auto Send Data to RDF");
        //Menu displayMenu = new Menu("Display");
        //MenuItem errorItem = new MenuItem("Error");
        //MenuItem warningItem = new MenuItem("Warning");
        //MenuItem infoItem = new MenuItem("Info");
        //MenuItem noneItem = new MenuItem("None");
        MenuItem exitItem = new MenuItem("Exit");

        //Add components to popup menu
        popup.add(aboutItem);
        popup.add(accountItem);
        popup.addSeparator();
        popup.add(visualItem);
        popup.add(browseDBItem);
        popup.add(statsItem);
        popup.addSeparator();
        popup.add(cb1);
        //popup.add(cb2);
        popup.addSeparator();
        //popup.add(displayMenu);
        //displayMenu.add(errorItem);
        //displayMenu.add(warningItem);
        //displayMenu.add(infoItem);
        //displayMenu.add(noneItem);
        popup.add(exitItem);

        Finals.trayIcon.setPopupMenu(popup);

        try {
            tray.add(Finals.trayIcon);
        } catch (AWTException e) {
            System.out.println("TrayIcon could not be added.");
            //Ubuntu -> http://askubuntu.com/questions/30742/how-do-i-access-and-enable-more-icons-to-be-in-the-system-tray
            return;
        }

//        Finals.trayIcon.addActionListener(new ActionListener() {
//            public void actionPerformed(ActionEvent e) {
//                //JOptionPane.showMessageDialog(null,"This dialog box is run from System Tray");
//                if (ab == null) {
//                    ab = new AboutJFrame();
//                }
//                RefineryUtilities.centerFrameOnScreen(ab);
//                ab.setVisible(true);
//            }
//        });
        
        aboutItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (ab == null) {
                    ab = new AboutJFrame();
                }
                RefineryUtilities.centerFrameOnScreen(ab);
                ab.setVisible(true);
            }
        });
        
        accountItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (su == null) {
                    su = new SetUserJFrame();
                }
                RefineryUtilities.centerFrameOnScreen(su);
                su.initiateValues();
                su.setVisible(true);
            }
        });

        browseDBItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                SwingUtilities.invokeLater(new Runnable() {
                    public void run() {
                        if (br == null) {
                            br = new DBBrowser();
                            br.createGUI(new File(Finals.dbURL), false, true);
                            br.showGui();
                        } else {
                            br.showGui();
                        }
                    }
                });
            }
        });

        visualItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (gb == null) {
                    gb = new GraphJFrame(db);
                }
                RefineryUtilities.centerFrameOnScreen(gb);
                gb.refreshResutls();
                gb.setVisible(true);
            }
        });
        
        statsItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (st == null) {
                    st = new StatisticsJFrame(db);
                }
                RefineryUtilities.centerFrameOnScreen(st);
                st.refreshResutls();
                st.setVisible(true);
            }
        });

        cb1.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                int cb1Id = e.getStateChange();
                
                //TODO: 8elei kolpa edw sto telos
                if (cb1Id == ItemEvent.SELECTED) {
                    Finals.stWithWindows = true;
                    //trayIcon.setImageAutoSize(true);
                } else {
                    Finals.stWithWindows = false;
                    //trayIcon.setImageAutoSize(false);
                }
            }
        });

//        cb2.addItemListener(new ItemListener() {
//            public void itemStateChanged(ItemEvent e) {
//                int cb2Id = e.getStateChange();
//                if (cb2Id == ItemEvent.SELECTED) {
//                    //trayIcon.setToolTip("Sun TrayIcon");
//                } else {
//                    //trayIcon.setToolTip(null);
//                }
//            }
//        });

//        ActionListener listener = new ActionListener() {
//            public void actionPerformed(ActionEvent e) {
//                MenuItem item = (MenuItem) e.getSource();
//                //TrayIcon.MessageType type = null;
//                System.out.println(item.getLabel());
//                if ("Error".equals(item.getLabel())) {
//                    //type = TrayIcon.MessageType.ERROR;
//                    trayIcon.displayMessage("Sun TrayIcon Demo",
//                            "This is an error message", TrayIcon.MessageType.ERROR);
//
//                } else if ("Warning".equals(item.getLabel())) {
//                    //type = TrayIcon.MessageType.WARNING;
//                    trayIcon.displayMessage("Sun TrayIcon Demo",
//                            "This is a warning message", TrayIcon.MessageType.WARNING);
//
//                } else if ("Info".equals(item.getLabel())) {
//                    //type = TrayIcon.MessageType.INFO;
//                    trayIcon.displayMessage("Sun TrayIcon Demo",
//                            "This is an info message", TrayIcon.MessageType.INFO);
//
//                } else if ("None".equals(item.getLabel())) {
//                    //type = TrayIcon.MessageType.NONE;
//                    trayIcon.displayMessage("Sun TrayIcon Demo",
//                            "This is an ordinary message", TrayIcon.MessageType.NONE);
//                }
//            }
//        };
//
//        errorItem.addActionListener(listener);
//        warningItem.addActionListener(listener);
//        infoItem.addActionListener(listener);
//        noneItem.addActionListener(listener);
        exitItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                tray.remove(Finals.trayIcon);
                wServer.doStop();
		scheduler.shutdownNow();
                Finals.saveProperties();
                db.closeConnecion();
                System.exit(0);
            }
        });
    }

}
