/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package sqlite;

import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Date;
import java.util.logging.Level;
import java.util.logging.Logger;
import sqlite.MySQLiteHelper;
import sqlite.dataSources.Categorization;
import sqlite.dataSources.Search;

/**
 *
 * @author drosatosgr
 */
public class CategorizationDataSource {

    MySQLiteHelper db;

    public CategorizationDataSource(MySQLiteHelper db) {
        this.db = db;
    }

    public boolean insertNewCategorization(Categorization ctz) {
        String sql = "INSERT INTO " + MySQLiteHelper.TABLE_CATEGORIZATION + " (" + MySQLiteHelper.CATEGORIZATION_COLUMN_SEARCH_ID + "," + MySQLiteHelper.CATEGORIZATION_COLUMN_CATEGORY_ID + "," + MySQLiteHelper.CATEGORIZATION_COLUMN_PERCENTAGE + ") VALUES (?, ?, ?);";
        return db.insert(sql, new Object[]{ctz.search_id, ctz.category_id, ctz.percentage});
    }

    public ArrayList<Categorization> getNonUploadedCategorizations() {
        ArrayList<Categorization> ret = new ArrayList<Categorization>();
        try {
            ResultSet rs = db.select("SELECT * FROM " + MySQLiteHelper.TABLE_CATEGORIZATION + " WHERE " + MySQLiteHelper.CATEGORIZATION_COLUMN_UPLOADED + "=?;", new Object[]{0});
            if (rs != null) {
                while (rs.next()) {
                    ret.add(new Categorization(rs));
                }
                rs.close();
            }
        } catch (SQLException se) {
            Logger.getLogger(CategorizationDataSource.class.getName()).log(Level.SEVERE, null, se);
        }
        return ret;
    }

    public void setCategorizationAsUploaded(Categorization c) {
        db.update("UPDATE " + MySQLiteHelper.TABLE_CATEGORIZATION + " SET " + MySQLiteHelper.CATEGORIZATION_COLUMN_UPLOADED + "=? Where " + MySQLiteHelper.CATEGORIZATION_COLUMN_ID + "=?;", new Object[]{c.uploaded, c._id});
    }
    
    public int getWeightOfCategory(Categorization c){
        int ret = 0;
        try {
            ResultSet rs = db.select("SELECT "+MySQLiteHelper.CATEGORIZATION_COLUMN_CATEGORY_ID+" FROM " + MySQLiteHelper.TABLE_CATEGORIZATION + " WHERE " + MySQLiteHelper.CATEGORIZATION_COLUMN_SEARCH_ID + "=? ORDER BY "+ MySQLiteHelper.CATEGORIZATION_COLUMN_PERCENTAGE+" ASC;", new Object[]{c.search_id});
            if (rs != null) {
                ArrayList<Integer> ids = new ArrayList<Integer>();
                 while (rs.next()) {
                     ids.add(rs.getInt(1));
                 }
                rs.close();
                return ids.indexOf(c.category_id) + 1;
            }
        } catch (SQLException se) {
            Logger.getLogger(CategorizationDataSource.class.getName()).log(Level.SEVERE, null, se);
        }
        return ret;
    }
    
    public ArrayList<Object> getInfoForRDF(Categorization c){
        ArrayList<Object> ret = new ArrayList<Object>();
        try {
            ResultSet rs = db.select("SELECT c.name as LeafName, s.date as Date FROM categorisation ct, search s, category c where ct.search_id=s._id and ct.category_id=c._id and ct._id = ? ;", new Object[]{c._id});
            if (rs != null) {
                 while (rs.next()) {
                     ret.add(rs.getString(1));
                     ret.add(rs.getLong(2));
                     ret.add(getWeightOfCategory(c));
                 }
                rs.close();
            }
        } catch (SQLException se) {
            Logger.getLogger(CategorizationDataSource.class.getName()).log(Level.SEVERE, null, se);
        }
        return ret;
    }

    public ArrayList<ArrayList<Object>> getDataForGraph(Date from, Date to, int topN) {
        ArrayList<ArrayList<Object>> ret = new ArrayList<ArrayList<Object>>();
        String sel = "SELECT ca._id as RootID,ca.name as RootName,c._id as LeafID,c.name as LeafName,ct.percentage as Weight, s._id as QueryID,s.query as Query FROM categorisation ct, search s, category c, category ca where ct.search_id=s._id and ct.category_id=c._id and c.parent_id=ca._id and s.date between ? and ? and ( select count(*) from categorisation cct where cct.percentage >= ct.percentage and cct.search_id = ct.search_id ) <= ?;";
        try {
            ResultSet rs = db.select(sel, new Object[]{from, to, topN});
            if (rs != null) {
                while (rs.next()) {
                    ArrayList<Object> obs = new ArrayList<Object>(7);
                    obs.add(rs.getInt(1));
                    obs.add(rs.getString(2));
                    obs.add(rs.getInt(3));
                    obs.add(rs.getString(4));
                    obs.add(rs.getDouble(5));
                    obs.add(rs.getInt(6));
                    obs.add(rs.getString(7));                    
                    ret.add(obs);
                }
                rs.close();
            }
        } catch (SQLException se) {
            Logger.getLogger(CategorizationDataSource.class.getName()).log(Level.SEVERE, null, se);
        }
        return ret;
    }
    
    public ArrayList<Object[]> getStatisticsOfCategorization(Date from, Date to) {
        //SELECT count(Distinct c.search_id), count(c.category_id) FROM categorisation c, search s where s.date between 0 and 1500000000000 and s._id = c.search_id and c.uploaded = 1 ;
        
        ArrayList<Object[]> ret = new ArrayList<Object[]>();
        try {
            ResultSet rs = db.select("SELECT count(Distinct c.search_id), count(c.category_id) FROM categorisation c, search s where s.date between ? and ? and s._id = c.search_id and c.uploaded = 1;", new Object[]{from,to});
            if (rs != null) {
                while (rs.next()) {
                    Object[] a = new Object[3];
                    a[0] = "Local Database";
                    a[1] = "Relevant Queries";
                    a[2] = rs.getInt(1);           
                    ret.add(a);
                    Object[] b = new Object[3];;
                    b[0] = "Private RDF";
                    b[1] = "Relevant Intentions";
                    b[2] = rs.getInt(2);           
                    ret.add(b);
                }
                rs.close();
            }
        } catch (SQLException se) {
            Logger.getLogger(SearchDataSource.class.getName()).log(Level.SEVERE, null, se);
        }
        return ret;
    }

}
