/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package sqlite;

import finals.Finals;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.sqlite.SQLiteConfig;
import tools.Files;

/**
 *
 * @author drosatosgr
 */
public class MySQLiteHelper {

    private static MySQLiteHelper mInstance = null;
    private static Connection conn = null;

    public static final String TABLE_SEARCH = "search";
    public static final String SEARCH_COLUMN_ID = "_id";
    public static final String SEARCH_COLUMN_QUERY = "query";
    public static final String SEARCH_COLUMN_ENGINE = "engine";
    public static final String SEARCH_COLUMN_DATE = "date";
    public static final String SEARCH_COLUMN_BROWSER = "browser";
    public static final String SEARCH_COLUMN_CATEGORISED = "categorised";
    private static final String CREATE_TABLE_SEARCH = "create table if not exists " + TABLE_SEARCH + "("
            + SEARCH_COLUMN_ID + " integer primary key autoincrement, "
            + SEARCH_COLUMN_QUERY + " text not null, "
            + SEARCH_COLUMN_ENGINE + " text not null, "
            + SEARCH_COLUMN_DATE + " integer not null, "
            + SEARCH_COLUMN_BROWSER + " text not null, "
            + SEARCH_COLUMN_CATEGORISED + " integer default 0 "
            + ");";

    public static final String TABLE_CATEGORY = "category";
    public static final String CATEGORY_COLUMN_ID = "_id";
    public static final String CATEGORY_COLUMN_NAME = "name";
    public static final String CATEGORY_COLUMN_DESCRIPTION = "description";
    public static final String CATEGORY_COLUMN_PARENT_ID = "parent_id";
    public static final String CATEGORY_COLUMN_FOR_UPLOADING = "for_uploading";
    private static final String CREATE_TABLE_CATEGORY = "create table if not exists " + TABLE_CATEGORY + "("
            + CATEGORY_COLUMN_ID + " integer primary key, "
            + CATEGORY_COLUMN_NAME + " text not null, "
            + CATEGORY_COLUMN_DESCRIPTION + " text, "
            + CATEGORY_COLUMN_PARENT_ID + " integer, "
            + CATEGORY_COLUMN_FOR_UPLOADING + " integer, "
            + " foreign key(" + CATEGORY_COLUMN_PARENT_ID + ") REFERENCES " + TABLE_CATEGORY + "(" + CATEGORY_COLUMN_ID + ") "
            + ");";

    public static final String TABLE_CATEGORIZATION = "categorisation";
    public static final String CATEGORIZATION_COLUMN_ID = "_id";
    public static final String CATEGORIZATION_COLUMN_SEARCH_ID = "search_id";
    public static final String CATEGORIZATION_COLUMN_CATEGORY_ID = "category_id";
    public static final String CATEGORIZATION_COLUMN_PERCENTAGE = "percentage";
    public static final String CATEGORIZATION_COLUMN_UPLOADED = "uploaded";
    private static final String CREATE_TABLE_CATEGORIZATION = "create table if not exists " + TABLE_CATEGORIZATION + "("
            + CATEGORIZATION_COLUMN_ID + " integer primary key autoincrement, "
            + CATEGORIZATION_COLUMN_SEARCH_ID + " integer not null, "
            + CATEGORIZATION_COLUMN_CATEGORY_ID + " integer not null, "
            + CATEGORIZATION_COLUMN_PERCENTAGE + " real not null, "
            + CATEGORIZATION_COLUMN_UPLOADED + " integer default 0, "
            + " unique("+CATEGORIZATION_COLUMN_SEARCH_ID+", "+CATEGORIZATION_COLUMN_CATEGORY_ID+"), "
            + " foreign key(" + CATEGORIZATION_COLUMN_SEARCH_ID + ") REFERENCES " + TABLE_SEARCH + "(" + SEARCH_COLUMN_ID + "), "
            + " foreign key(" + CATEGORIZATION_COLUMN_CATEGORY_ID + ") REFERENCES " + TABLE_CATEGORY + "(" + CATEGORY_COLUMN_ID + ") "
            + ");";

    private MySQLiteHelper() {
        mInstance = this;
    }

    public static MySQLiteHelper getInstance() {
        if (mInstance == null) {
            mInstance = new MySQLiteHelper();
            openConnecion();
        }
        return mInstance;
    }

    private static void openConnecion() {
        if (conn == null) {
            try {
                if (!Files.existsFile(Finals.dbFolder)) {
                    Files.createDirectory(Finals.dbFolder);
                }
                boolean exist = Files.existsFile(Finals.dbURL);
                Class.forName("org.sqlite.JDBC");
                SQLiteConfig config = new SQLiteConfig();
                config.setEncoding(SQLiteConfig.Encoding.UTF8);
                config.enforceForeignKeys(true);
                conn = DriverManager.getConnection("jdbc:sqlite:" + Finals.dbURL, config.toProperties());
                conn.setAutoCommit(true);
                if (!exist) {
                    createTables();
                }
            } catch (ClassNotFoundException ex) {
                Logger.getLogger(MySQLiteHelper.class.getName()).log(Level.SEVERE, null, ex);
            } catch (SQLException ex) {
                Logger.getLogger(MySQLiteHelper.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    public void setAutoCommit(boolean value) {
        if (conn != null) {
            try {
                conn.setAutoCommit(value);
            } catch (SQLException ex) {
                Logger.getLogger(MySQLiteHelper.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    public synchronized void closeConnecion() {
        if (mInstance != null && conn != null) {
            try {
                conn.close();
                mInstance = null;
                conn = null;
            } catch (SQLException ex) {
                Logger.getLogger(MySQLiteHelper.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    private static void createTables() {
        if (mInstance != null && conn != null) {
            try {
                Statement stmt = conn.createStatement();

                stmt.executeUpdate(CREATE_TABLE_SEARCH);
                stmt.executeUpdate(CREATE_TABLE_CATEGORY);
                stmt.executeUpdate(CREATE_TABLE_CATEGORIZATION);

                (new CategoryDataSource(mInstance)).loadCategoriesFromCSV(Finals.categoriesCSV);

                stmt.close();
            } catch (SQLException ex) {
                Logger.getLogger(MySQLiteHelper.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    public Connection getConnectionWithDB() {
        return conn;
    }

    public void dropTables() {
        if (mInstance != null && conn != null) {
            try {
                Statement stmt = conn.createStatement();

                stmt.executeUpdate("DROP TABLE IF EXISTS " + TABLE_SEARCH + ";");
                stmt.executeUpdate("DROP TABLE IF EXISTS " + TABLE_CATEGORY + ";");
                stmt.executeUpdate("DROP TABLE IF EXISTS " + TABLE_CATEGORIZATION + ";");

                stmt.close();
            } catch (SQLException ex) {
                Logger.getLogger(MySQLiteHelper.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    public boolean insert(String str, Object[] val) {
        boolean ret = false;
        if (conn != null) {
            try {
                PreparedStatement ps;
                ps = conn.prepareStatement(str);
                for (int i = 0; i < val.length; i++) {
                    ps.setObject(i + 1, val[i]);
                }
                ps.executeUpdate();
                ps.close();
                ret = true;
            } catch (SQLException se) {
                Logger.getLogger(MySQLiteHelper.class.getName()).log(Level.SEVERE, null, se);
            }
        }
        return ret;
    }

    public boolean update(String str, Object[] val) {
        boolean ret = false;
        if (conn != null) {
            try {
                PreparedStatement ps;
                ps = conn.prepareStatement(str);
                for (int i = 0; i < val.length; i++) {
                    ps.setObject(i + 1, val[i]);
                }
                ps.executeUpdate();
                ps.close();
                ret = true;
            } catch (SQLException se) {
                Logger.getLogger(MySQLiteHelper.class.getName()).log(Level.SEVERE, null, se);
            }
        }
        return ret;
    }

    public boolean delete(String str) {
        boolean ret = false;
        if (conn != null) {
            try {
                Statement s = conn.createStatement();
                s.executeUpdate(str);
                s.close();
                ret = true;
            } catch (SQLException se) {
                Logger.getLogger(MySQLiteHelper.class.getName()).log(Level.SEVERE, null, se);
            }
        }
        return ret;
    }

    public ResultSet select(String str, Object[] val) {
        ResultSet rs = null;
        if (conn != null) {
            try {
                PreparedStatement ps;
                ps = conn.prepareStatement(str);
                if (val != null) {
                    for (int i = 0; i < val.length; i++) {
                        ps.setObject(i + 1, val[i]);
                    }
                }
                rs = ps.executeQuery();
                return rs;
            } catch (SQLException se) {
                Logger.getLogger(MySQLiteHelper.class.getName()).log(Level.SEVERE, null, se);
            }
        }
        return rs;

    }

}
