package tools;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.Window;

public class RefineryUtilities {

	public static void centerFrameOnScreen(final Window frame) {
		positionFrameOnScreen(frame, 0.5, 0.5);
	}

	/**
	 * Positions the specified frame at a relative position in the screen, where 50% is considered
	 * to be the center of the screen.
	 *
	 * @param frame  the frame.
	 * @param horizontalPercent  the relative horizontal position of the frame (0.0 to 1.0,
	 *                           where 0.5 is the center of the screen).
	 * @param verticalPercent  the relative vertical position of the frame (0.0 to 1.0, where
	 *                         0.5 is the center of the screen).
	 */
	public static void positionFrameOnScreen(final Window frame,
			final double horizontalPercent,
			final double verticalPercent) {

		final Dimension s = Toolkit.getDefaultToolkit().getScreenSize();
		final Dimension f = frame.getSize();
		final int w = Math.max(s.width - f.width, 0);
		final int h = Math.max(s.height - f.height, 0);
		final int x = (int) (horizontalPercent * w);
		final int y = (int) (verticalPercent * h);
		frame.setBounds(x, y, f.width, f.height);

	}


	public static void positionChildFrameOnCenterOfParentFrame(final Window Parentframe, final Window Childframe) {
		final int p_x = Parentframe.getX();
		final int p_y = Parentframe.getY();
		final Dimension p_f = Parentframe.getSize();
		final Dimension c_f = Childframe.getSize();
		final int w = p_f.width - c_f.width;
		final int h = p_f.height - c_f.height;
		final int x = (int) (0.5 * w);
		final int y = (int) (0.5 * h);
		Childframe.setBounds(x+p_x, y+p_y, c_f.width, c_f.height);
	}
}
